"""Tests for systems.json files
"""

import unittest
import json
import os.path

# 3rd Party imports
from jsonschema import validate


class TestSystemSyntax(unittest.TestCase):
    """Validates all plugins in `api/plugins/` using our JSON Schema.
    """
    # Get file paths relative to this file's known location
    schema_path = os.path.join(os.path.dirname(__file__), "system-schema.json")
    system_files = [os.path.join(os.path.dirname(__file__), "../systems.json")]
    # Class variable holding the current JSON Schema. See setUpClass() for more info
    schema = None

    @classmethod
    def setUpClass(cls):
        """Open and read in the current plugin JSON Schema from disk for us during testing.
        """
        with open(cls.schema_path, mode='r') as f:
            cls.schema = json.load(f)
        cls.errors = []

    def tearDown(self):
        if len(self.errors) > 0:
            for err in self.errors:
                print("-"*70)
                try:
                    msg = err["error"].message
                except AttributeError:
                    msg = err["error"]
                print(f"Error validating system '{err['system']}': {msg} -- {err['file']}")
            self.assertIsNone(1)

    def test_isvalid(self):
        """For each systems file, validate each system in file against our schema. If an
        exception is raised, the plugin file is NOT valid.
        """
        for system_file in self.system_files:
            with open(system_file, mode='r') as f:
                systems = json.load(f).get("systems")
                if systems:
                    for system, system_data in systems.items():
                        try:
                            self.assertIsNone(validate(instance=system_data, schema=self.schema))
                        except Exception as e:
                            self.errors.append({"system": system, "file": os.path.basename(system_file), "error": e})
                else:
                    self.errors[system_file] = "Has no systems attribute."

